import zlib

from fastapi import APIRouter, Depends, HTTPException
from pydantic import ValidationError
from starlette import status
from starlette.responses import Response

from id_broker_common.kelvin import kelvin_session
from provisioning_plugin.dependencies import check_for_authority_admin, get_school_by_name
from provisioning_plugin.models import NonEmptyStr, NoStarStr, School
from ucsschool.kelvin.client import InvalidRequest, School as KelvinSchool, Session

router = APIRouter(tags=["schools"])


@router.get("/schools/{name}", response_model=School)
async def get(
    name: NoStarStr,
    school_authority: NonEmptyStr,
    school: KelvinSchool = Depends(get_school_by_name),
) -> School:
    try:
        return School.without_prefix(School.from_kelvin_school(school.as_dict()), school_authority)
    except (ValidationError, ValueError) as exc:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"The school {name}" f"in school_authority {school_authority} is malformed!",
        ) from exc


@router.head("/schools/{name}")
async def get_head(
    name: NoStarStr,
    school_authority: NonEmptyStr,
    school: KelvinSchool = Depends(get_school_by_name),
):
    await get(name, school_authority, school)
    return Response(status_code=status.HTTP_200_OK)


@router.post("/schools", response_model=School, status_code=201)
async def post(
    school_authority: str,
    school_data: School,
    session: Session = Depends(kelvin_session),
    _is_admin: bool = Depends(check_for_authority_admin),
):
    school_data = school_data.with_prefix(school_data, school_authority)
    # Avoid automatic generation of dc_name by setting it explicitly.
    # The values are not used by the ID-Broker.
    # Schools of the same authority have the same dc_name.
    dc_name = f"dc-{zlib.crc32(school_authority.encode('UTF-8'))}"
    school = KelvinSchool(
        name=school_data.name,
        display_name=school_data.display_name,
        home_share_file_server=dc_name,
        class_share_file_server=dc_name,
        educational_servers=[dc_name],
        session=session,
    )
    try:
        await school.save()
    except InvalidRequest as exc:
        raise HTTPException(status_code=exc.status, detail=exc.reason) from exc
    return School.without_prefix(School.from_kelvin_school(school.as_dict()), school_authority)
