import asyncio
import os
import random
import warnings
import zlib
from typing import Dict, List

import pytest

from id_broker_common.kelvin import kelvin_session, kelvin_session_kwargs, load_kelvin_session_kwargs
from provisioning_plugin.plugin import PLUGIN_SETTINGS_FILE
from ucsschool.kelvin.client import (
    InvalidRequest,
    NoObject,
    School as KelvinSchool,
    SchoolClass as KelvinSchoolClass,
    User as KelvinUser,
    UserResource,
)

from .utils import DEFAULT_PASSWORD, DEFAULT_SCHOOL_AUTHORITY, DEFAULT_SCHOOL_NAME, get_access_token


@pytest.fixture(scope="function", autouse=True)
def faker_seed():
    return random.randint(0, 10000)


@pytest.fixture(scope="session")
def event_loop():
    loop = asyncio.get_event_loop()
    yield loop
    loop.close()


@pytest.fixture()
def create_school(kelvin_session_obj):
    async def _create_school(
        school_authority: str = DEFAULT_SCHOOL_AUTHORITY,
        name: str = DEFAULT_SCHOOL_NAME,
        display_name: str = None,
    ):
        if display_name is None:
            display_name = name
        school = KelvinSchool(
            name=f"{school_authority}-{name}",
            display_name=display_name,
            session=kelvin_session_obj,
            educational_servers=[f"dc-{zlib.crc32(school_authority.encode('UTF-8'))}"],
        )
        try:
            await school.save()
        except InvalidRequest as exc:
            if exc.status != 409:
                raise exc
        await school.reload()
        return school

    return _create_school


@pytest.fixture(scope="function")
async def user_cleaner(kelvin_session_obj):
    users = []

    def _user_cleaner(record_uid, source_uid):
        users.append((record_uid, source_uid))

    yield _user_cleaner

    for record, source in users:
        searched_users = [
            user
            async for user in UserResource(session=kelvin_session_obj).search(
                record_uid=record, source_uid=source
            )
        ]
        if len(searched_users) == 0:
            warnings.warn(f"The user {record}, {source} could not be found and thus was not cleaned up.")
        for kelvin_user in searched_users:
            await kelvin_user.delete()


@pytest.fixture()
def create_user(kelvin_session_obj, faker, user_cleaner):
    async def _create_user(
        user_name: str = None,
        password: str = DEFAULT_PASSWORD,
        first_name: str = None,
        last_name: str = None,
        school_authority: str = DEFAULT_SCHOOL_AUTHORITY,
        school: str = DEFAULT_SCHOOL_NAME,
        roles: List[str] = None,
        school_classes: Dict[str, List[str]] = None,
    ):
        if user_name is None:
            user_name = faker.pystr(1, 7)
        if first_name is None:
            first_name = faker.first_name()
        if last_name is None:
            last_name = faker.last_name()
        if school_classes is None:
            school_classes = {}
        if roles is None:
            roles = ["teacher"]
        _user_name = (
            user_name
            if user_name == f"provisioning-{school_authority}"
            else f"{school_authority}-{user_name}"
        )
        user = KelvinUser(
            firstname=first_name,
            lastname=last_name,
            name=_user_name,
            school=f"{school_authority}-{school}",
            schools=[f"{school_authority}-{school}"],
            roles=roles,
            password=password,
            source_uid=f"IDBROKER-{school_authority}",
            record_uid=f"record_{_user_name}",
            session=kelvin_session_obj,
            school_classes={
                f"{school_authority}-{school}": school_classes
                for school, school_classes in school_classes.items()
            },
        )
        try:
            await user.save()
        except InvalidRequest as exc:
            if exc.status != 409:
                raise exc
            warnings.warn(f"recycling existing user {user.record_uid} {user.source_uid}")
        user_cleaner(user.record_uid, user.source_uid)
        return user

    return _create_user


@pytest.fixture()
async def create_school_class(kelvin_session_obj, faker):
    created_school_classes = []

    async def _create_school_class(
        school_authority: str = DEFAULT_SCHOOL_AUTHORITY,
        name: str = None,
        school: str = DEFAULT_SCHOOL_NAME,
        descr: str = "",
        members: List[str] = None,
    ):
        if name is None:
            name = faker.pystr(1, 10)
        if members is None:
            members = []
        school_class = KelvinSchoolClass(
            name=name,
            school=f"{school_authority}-{school}",
            description=descr,
            users=members,
            session=kelvin_session_obj,
        )
        await school_class.save()
        await school_class.reload()
        created_school_classes.append(school_class)
        return school_class

    yield _create_school_class

    for school_class in created_school_classes:
        try:
            await school_class.delete()
        except NoObject:
            pass


@pytest.fixture(scope="session")
def url_fragment():
    return f"http://{os.environ['DOCKER_HOST_NAME']}/ucsschool/apis"


@pytest.fixture()
async def kelvin_session_obj():
    kelvin_session_kwargs.update(load_kelvin_session_kwargs(PLUGIN_SETTINGS_FILE))
    session = await kelvin_session()
    return session


@pytest.fixture()
def auth_headers(create_user, create_school):
    async def _auth_headers(school_authority: str):
        await create_user(
            user_name=f"provisioning-{school_authority}", school_authority=school_authority
        )
        token = get_access_token(f"provisioning-{school_authority}")
        return {"Authorization": f"Bearer {token}"}

    return _auth_headers
