import pytest
from fastapi import HTTPException

from provisioning_plugin.utils import (
    get_roles_for_school,
    get_source_uid_for_authority,
    remove_prefix,
    search_kelvin_obj,
)
from ucsschool.kelvin.client import InvalidRequest


class AsyncIteratorMock:
    def __init__(self, seq):
        self.iter = iter(seq)

    def __aiter__(self):
        return self

    async def __anext__(self):
        try:
            return next(self.iter)
        except StopIteration:
            raise StopAsyncIteration


@pytest.mark.asyncio
async def test_search_kelvin_obj(mocker):
    resource_mock = mocker.Mock()
    resource_mock.search = mocker.Mock(return_value=AsyncIteratorMock([1]))
    result = await search_kelvin_obj("TEST", resource_mock, name="a", uid=2)
    assert result == 1
    resource_mock.search.assert_called_with(name="a", uid=2)


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "return_value,status_code,exc_type", [([], 404, HTTPException), ([1, 2], 500, HTTPException)]
)
async def test_search_kelvin_obj_http_exception(return_value, status_code, exc_type, mocker):
    resource_mock = mocker.Mock()
    resource_mock.Meta.kelvin_object.__name__ = "Resource"
    resource_mock.search = mocker.Mock(return_value=AsyncIteratorMock(return_value))
    with pytest.raises(exc_type) as exc_info:
        await search_kelvin_obj("TEST", resource_mock)
    assert exc_info.type == exc_type
    assert exc_info.value.status_code == status_code


@pytest.mark.asyncio
async def test_search_kelvin_obj_invalid_request(mocker):
    resource_mock = mocker.Mock()
    resource_mock.search = mocker.Mock(side_effect=InvalidRequest(status=422, reason="Unprocessable"))
    with pytest.raises(HTTPException) as exc_info:
        await search_kelvin_obj("TEST", resource_mock)
    assert exc_info.type == HTTPException
    assert exc_info.value.status_code == 422
    assert exc_info.value.detail == "Unprocessable"


@pytest.mark.parametrize(
    "prefix,value,expected",
    [
        ("TEST-", "TEST-string", "string"),
        ("OTHER-PRE-FIX", "OTHER-PRE-FIX-FIX-FIX", "-FIX-FIX"),
        ("Test-", "Test-", ""),
        ("", "TEST-string", "TEST-string"),
    ],
)
def test_remove_school_authority_prefix(prefix, value, expected):
    assert remove_prefix(value, prefix) == expected


def test_remove_school_authority_prefix_exc():
    with pytest.raises(ValueError):
        remove_prefix("RANDOM_STR", "PREFIX-")


@pytest.mark.parametrize(
    "roles,expected",
    [
        ([], set()),
        (["teacher:school:DEMOSCHOOL"], {"teacher"}),
        [["student:school:DEMOSCHOOL", "teacher:school:OTHERSCHOOL"], {"student"}],
        (
            ["teacher:school:DEMOSCHOOL", "staff:school:DEMOSCHOOL", "student:school:OTHERSCHOOL"],
            {"teacher", "staff"},
        ),
        (["teacher:other:DEMOSCHOOL"], set()),
        (["teacher:school:DEMOSCHOOL".upper()], set()),
    ],
)
def test_get_roles_for_school(roles, expected):
    assert get_roles_for_school(roles, "DEMOSCHOOL") == expected


@pytest.mark.parametrize("role", ["", ":", "::", ":::", "teacher:school:"])
def test_get_roles_for_schools_exc(role):
    with pytest.raises(ValueError):
        get_roles_for_school([role], "DEMOSCHOOL")


@pytest.mark.parametrize(
    "school_authority,expected",
    [
        ("TEST", "IDBROKER-TEST"),
        ("OTHER-PRE-FIX", "IDBROKER-OTHER-PRE-FIX"),
        ("OTHER-pre-FIX8", "IDBROKER-OTHER-pre-FIX8"),
    ],
)
def test_get_source_uid_for_authority(school_authority, expected):
    assert get_source_uid_for_authority(school_authority) == expected
